from Screen import Screen
import loader
import os
from Config import *
from pygame.locals import *
import pygame
import math

from CloudManager import CloudManager
from InfoManager import InfoManager
from Doom import Doom
from MapManager import MapManager
from PhysicsManager import PhysicsManager
from Player import Player
from HelpText import HelpText
from State import State
from Rope import Rope
from Button import Button
from SoundManager import SoundManager
from LevelManager import LevelManager
from VictoryScreen import VictoryScreen
from TitleScreen import TitleScreen

class GameScreen(Screen):
	def __init__(self):
		self.state = State()
		self.state.init_state()
		self.state.quality = 1
		self.loader = loader.loader()
		self.state.fullscreen = False
		self.state.screensize = ScreenSize
		self.state.sound = SoundManager()
		self.state.reset_count = 0
		self.background = pygame.transform.scale(self.loader.load(os.path.join("res","art","gradient.png"))[0],(ScreenSize[0],ScreenSize[1]-100))
		
		self.physics = PhysicsManager()
		self.clouds = CloudManager(3)
		self.info = InfoManager()
		self.map = MapManager(self.physics.space)
		self.state.map = self.map
		self.levels = LevelManager(self.map)
		self.state.levels = self.levels
		self.help = HelpText([self.map])
		self.doom = Doom(self.physics.space,self.reset)
		self.player = Player(self.physics.space,self.map.width)
		self.rope = Rope()
		self.physics.map = self.map
		self.physics.reset_func = self.show_victory
		self.physics.setup_collisions()
		self.x = 0
		self.scroll = [0,0]
		self.cursor = None # (surface, index) for placeable objects
		self.mouse_coords = None
		self.buttons = [ Button("quit",(35,0),self.quit), Button("reset",(self.state.screensize[0]-100,0),self.reset), Button("help",(self.state.screensize[0]-200,-10),self.toggle_help) ]
		self.victory = VictoryScreen()
		self.state.in_victory = False
		self.titlescreen = TitleScreen()
		self.reset()
		
		# Tutorial text
		self.show_tutorial_text = False
		font = self.loader.load_font("slkscr",16)
		messages = ["Put the Ball in the Bucket",None,
			"All you have to do is put the ball in the bucket",None,
			"Build the level with the blocks in the toolbar",None,
			"Click blocks in the toolbar, then","click on the stage to place them.",None,
			"When you are ready","Press 'space' to drop the ball",None,
			"'f' to toggle fullscreen","'q' to toggle draw mode","'Escape' resets the level"]
		self.tutorial_text = pygame.Surface((500,16*len(messages)))
		self.tutorial_text.set_colorkey((0,0,0))
		self.tutorial_coords = (self.state.screensize[0]/2-250,50)
		y = 0
		for msg in messages:
			if msg:
				txt = font.render(msg,False,(64,64,64))
				self.tutorial_text.blit(txt,(250-txt.get_size()[0]/2,y))
			y += 16
	
	def toggle_help(self):
		self.show_tutorial_text = not self.show_tutorial_text
	
	def quit(self):
		raise SystemExit(0)
	
	def update(self):
		self.state.sound.update()
		self.clouds.update(self.x)
		self.info.update()
		self.doom.update()
		self.map.update()
		self.physics.update()
		self.player.update()
		self.help.update()
		if self.player.player:
			x = self.player.position()[0] - ScreenSize[0]/2
			if x > 0 and x < self.map.width - ScreenSize[0]:
				self.x = x
		return True
	
	def next_level(self):
		self.levels.next_level()
		self.state.reset_count = 0
		self.reset()
	
	def show_victory(self):
		self.state.in_victory = True
		self.victory.render()
	
	def reset(self):
		self.state.in_victory = False
		self.map.empty()
		self.rope = Rope()
		self.player.kill()
		self.doom.reset()
		self.help.reset()
		self.levels.setup_current_level()
		self.state.reset_count += 1
		
	def draw(self,surf):
		surf.fill((230,230,230))
		if self.state.quality > 0:
			surf.blit(self.background,(0,0))
		self.help.draw(surf)
		for button in self.buttons:
			button.draw(surf)
		if self.show_tutorial_text:
			surf.blit(self.tutorial_text,self.tutorial_coords)
		self.clouds.draw(surf,self.x)
		self.map.draw(surf,self.x)
		if self.rope:
			self.rope.draw(surf,self.x)
		self.player.draw(surf,self.x)
		self.doom.draw(surf,self.x)
		self.info.draw(surf)
		if self.cursor:
			surf.blit(self.cursor[0],(self.mouse_coords[0]-self.cursor[0].get_size()[0]/2,self.mouse_coords[1]-self.cursor[0].get_size()[1]/2))
		if self.state.in_victory:
			self.victory.draw(surf)
		if self.titlescreen:
			self.titlescreen.draw(surf)
	
	def handle_event(self,event):
		if event.type == KEYDOWN:
#			if event.key == pygame.K_LEFT:
#				self.player.go(0)
#			elif event.key == pygame.K_RIGHT:
#				self.player.go(1)
#			elif event.key == pygame.K_UP:
#				self.player.go(2)
#			elif event.key == pygame.K_DOWN:
#				self.player.go(3)
			if event.key == pygame.K_ESCAPE:
				if not self.rope:
					self.reset()
			elif event.key == 119: # 'w'
				self
			elif event.key == 102: # 'f'
				opts = 0
				if DoubleBuffer:
					opts = opts | pygame.DOUBLEBUF
				self.state.fullscreen = not self.state.fullscreen
				if self.state.fullscreen:
					opts = opts | pygame.FULLSCREEN
				screen = pygame.display.set_mode(ScreenSize,opts)
			elif event.key == 32: # space
				if self.rope:
					self.player.spawn(self.rope.spawn)
					self.rope = None
				else:
					if self.state.in_victory:
						self.next_level()
						self.state.in_victory = False
					else:
						self.reset()
			elif event.key == 113:
				self.state.quality += 1
				if self.state.quality > 1:
					self.state.quality = 0
		elif event.type == MOUSEMOTION:
			self.mouse_coords = event.pos
			for button in self.buttons:
				button.hover(event.pos)
			self.info.hover(event.pos)
		elif event.type == MOUSEBUTTONDOWN:
			if self.titlescreen:
				self.titlescreen = None
			if self.state.in_victory:
				self.state.in_victory = False
				self.next_level()
				return True
			for button in self.buttons:
				if button.click(event.pos):
					return True
			res = self.info.click(event.pos)
			if res:
				self.cursor = res
			else:
				if self.cursor:
					self.map.spawn(self.cursor[1],event.pos)
				self.cursor = None
		return True